/* Rotator.m				 
 *
 * Rotates the world. Moves the camera. Will change a lot!!
 *
 * For more interface-info see the header file. More in depth information
 * can be found in the source-code.
 *
 * Written by: 		Thomas Engel
 * Created:    		22.12.1993 (Copyleft)
 * Last modified: 	12.04.1994
 */

#import "Rotator.h"
#import "../BeakerCamera.h"
#import "BBSlider.h"
#import "Misc3DDeviceServer.h"
#import "Misc3DDeviceDriver.h"
#import <remote/NXProxy.h>
#import "MiscRtMatrix.h"

@implementation Rotator

- makeKeyAndOrderFront:sender
{
	// If there hasnt been a Rotator-Panel yet. Lets do some init stuff.
	
	if( !panel )
	{
		// To get the right Slider behaviour we will use our own slider class.
	
		[BBSlider poseAs:[Slider class]];

		if( [NXApp loadNibSection:"Rotator.nib" owner:self] == nil )
			NXRunAlertPanel( NULL, "Couldn't load Rotator.nib",
							 "OK", NULL, NULL );
	}
	[panel makeKeyAndOrderFront:self];
	return self;
}

- setTarget:anObject
{
	target = anObject;
	return self;
}

- setCamera:anObject
{
	camera = anObject;
	return self;
}

- connectTo3DMouse:sender
{
	id	ourServer;

	// We will connect to the server and launcg our private dispatcher
	// It just works.
		
	server3D = [NXConnection connectToName:"localhost/3DDeviceServer"];
	ourServer = [server3D connectionForProxy];
	[ourServer runFromAppKit];
	
	mouse3D = [server3D mouse];
	[mouse3D setProtocolForProxy:@protocol(Misc3DDeviceDriverProtocol)];
	[mouse3D setTarget:self];
	[mouse3D enableEvents];
	[mouse3D setUseExternalSync:YES];

	return self;
}

- xRotation:sender;
{
	float 	angle;
	RtPoint xAxis = {1.0,0.0,0.0};
	
	angle = [sender floatValue] - lastSliderValue;
	[rotValue setIntValue:[sender intValue]];
	
	if( angle != 0.0 )
	{
		[target rotateAngle:angle axis:xAxis];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- yRotation:sender;
{
	float 	angle;
	RtPoint yAxis = { 0.0, 1.0, 0.0 };
	
	angle = [sender floatValue] - lastSliderValue;
	[rotValue setIntValue:[sender intValue]];
	
	if( angle != 0.0 )
	{
		[target rotateAngle:angle axis:yAxis];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- zRotation:sender;
{
	float 	angle;
	RtPoint zAxis = { 0.0, 0.0, 1.0 };
	
	angle = [sender floatValue] - lastSliderValue;
	[rotValue setIntValue:[sender intValue]];
	
	if( angle != 0.0 )
	{
		[target rotateAngle:angle axis:zAxis];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- xTranslation:sender
{
	// xTranslation will move our camera AND its toPoint across the xAxis.

	RtPoint fromPoint;
	RtPoint toPoint;
	float	angle;
	float 	delta;
	
	delta = ( [sender floatValue] - lastSliderValue );
	[transValue setIntValue:[sender intValue]];
	
	if( delta != 0.0 )
	{
		[camera getEyeAt:&fromPoint toward:&toPoint roll:&angle];
		fromPoint[0] -= delta;
		toPoint[0] -= delta;
		[camera setEyeAt:fromPoint toward:toPoint roll:angle];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- yTranslation:sender
{
	// yTranslation will move our camera AND its toPoint across the yAxis.
	
	RtPoint fromPoint;
	RtPoint toPoint;
	float	angle;
	float 	delta;
	
	delta = ( [sender floatValue] - lastSliderValue );
	[transValue setIntValue:[sender intValue]];
	
	if( delta != 0.0 )
	{
		[camera getEyeAt:&fromPoint toward:&toPoint roll:&angle];
		fromPoint[1] -= delta;
		toPoint[1] -= delta;
		[camera setEyeAt:fromPoint toward:toPoint roll:angle];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- zTranslation:sender
{
	// zTranslation will ONLY move our camera across the zAxis.
	// So our focus will always remain at z=0 !!!
	
	RtPoint fromPoint;
	RtPoint toPoint;
	float	angle;
	float 	delta;
	
	delta = ( [sender floatValue] - lastSliderValue );
	[transValue setIntValue:[sender intValue]];
	
	if( delta != 0.0 )
	{
		[camera getEyeAt:&fromPoint toward:&toPoint roll:&angle];
		fromPoint[2] -= delta;
		[camera setEyeAt:fromPoint toward:toPoint roll:angle];
		[camera display];
	}
	lastSliderValue = [sender floatValue];
	return self;
}

- benchMark:sender
{
	// We will use this method to trigger a redraw and then to estimate the
	// time consumpt.
	
	struct timezone tzone;
	struct timeval	realtime1;
	struct timeval	realtime2;
	double	myTime;
	
	gettimeofday(&realtime1,&tzone);
	[camera display];
	NXPing();
	gettimeofday(&realtime2,&tzone);
	
	myTime = ((realtime2.tv_sec + realtime2.tv_usec / 1.0E6 ) -
			  (realtime1.tv_sec + realtime1.tv_usec / 1.0E6 ));
	
	[benchTime setDoubleValue:myTime];
	return self;
}

- fastStyle:sender
{
	[self transformationWillStart:self];
	return self;
}

- niceStyle:sender
{
	[self transformationDidEnd:self];
	return self;
}

/*
 * Here comes the part where we implement the Misc3DMouseTarget protocol.
 * Together with the connectTo3DMouse method it provides the whole bunch of
 * code necessary to support any kind of 3D mouse.
 */

- (BOOL)worldIsRightHanded
{
	return NO;
}

- (oneway void)transformationEvent:(bycopy id)aMatrix
{
	// This method tries to aply the 3dmouse matrix to our data.
	// This method should ONLY be use by the 3DDriver mouse !!
	// Otherwise there might be problems with the sync. (I could solve
	// that by using som kind of internal flag...but..this object will need
	// a complete redesign anyway..)
	//
	// Only the rotation will be applied to the mol. Translation will be
	// extracted and set seperatly at our camera.
	// This is not perfect in any way. But it should be enough for a while.
	//
	// There is a problem when move the camera to the positiv z area.
	// The the transformation won't fit the reality. This is because of our
	// 'hacked' rotation stuff.
	// I should really apply the rotation to the world this camera does see.
	// This is not a bug in the 3DDeviceServer. Remember that you always to
	// transformation from the cameras point of view!! And the camera should be
	// on the negativ z-axis in a left-handed world; and on the positiv z-axis
	// in a right handed world!!
	
	RtMatrix	m;
	float		x, y, z;
	RtPoint 	fromPoint;
	RtPoint 	toPoint;
	float		angle;

	[aMatrix getTransformMatrix:m];
	
	x = -m[3][0];
	y = -m[3][1];
	z = -m[3][2];	// This is because the camera has to move differently
					// to show us the intended move of the object!
					// This will change later.
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;

	if( x != 0 || y != 0 || z != 0 )
	{
		[camera getEyeAt:&fromPoint toward:&toPoint roll:&angle];
		fromPoint[0] += x;
		fromPoint[1] += y;
		fromPoint[2] += z;
		toPoint[0] += x;
		toPoint[1] += y;
		[camera setEyeAt:fromPoint toward:toPoint roll:angle];
	}

	if( [aMatrix doesRotate] )
		[target concatTransformMatrix:m premultiply:NO];
		
	[camera display];
	[mouse3D syncEvents];
	return;
}

- (oneway void)transformationWillStart:sender
{
	[[camera worldShape] rotationStyleOn:YES];
	[camera setHider:N3D_InOrderRendering];
	[camera display];
	return;
}

- (oneway void)transformationDidEnd:sender
{
	// If the sender is one of our sliders we will reset it to the
	// Zero point.
	
	if( sender == aSlider ||
		sender == bSlider ||
		sender == cSlider )
	{
		[sender setFloatValue:0];
		[rotValue setIntValue:0];
		lastSliderValue = 0;
	}
	else if( sender == xSlider ||
			 sender == ySlider ||
			 sender == zSlider )
	{
		[sender setFloatValue:0];
		[transValue setIntValue:0];
		lastSliderValue = 0;
	}
	
	[[camera worldShape] rotationStyleOn:NO];
	[camera setHider:N3D_HiddenRendering];
	[camera display];
	return;
}

- (oneway void)keyEvent:(char *)theKeys
{
	return;
}

@end

/*
 * History: 12.04.94 Included 3DDeviceServer support.
 *
 *			15.01.94 Added the zoom mode.
 *
 *			22.12.93 First code written.
 *
 *
 * Bugs: yes
 */