/*
 * KPC488 driver interface
 * Copyright (C) 1994  W. Eric Norum
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * W. Eric Norum
 * Saskatchewan Accelerator Laboratory
 * University of Saskatchewan
 * 107 North Road
 * Saskatoon, Saskatchewan, CANADA
 * S7N 5C6
 * 
 * eric@skatter.usask.ca
 */

#import <driverkit/IODevice.h>
#import <driverkit/IODirectDevice.h>
#import <driverkit/i386/directDevice.h>
#import <driverkit/i386/IOEISADeviceDescription.h>
#import <driverkit/IODeviceDescription.h>
#import <driverkit/IOConfigTable.h>
#import <machkit/NXLock.h>
#import <mach/message.h>
#import <kernserv/ns_timer.h>

@interface KPC488:IODirectDevice
{
@public
	/*
	 * Device I/O ports
	 */
	IORange 				isaPorts;

	/*
	 * DMA buffer
	 * Use a local buffer since DMA to/from user space
	 * using host-master DMA is just too painful.
	 */
	char					*dmaBuffer;
	IOEISADMABuffer			ioEISADmaBuffer;
	unsigned int			dmaBufferPhysicalAddress;
	unsigned int			dmaLength;

	/*
	 * DMA pseudo-interrupt
	 */
	msg_header_t			dmaInterruptMessage;
	ns_time_t				dmaCheckInterval;
	BOOL					checkingDMA;

	/*
	 * IEEE-488 devices opened
	 */
	unsigned long			openFlags;
	int						myAddress;

	/*
	 * Interthread command passing
	 */
	NXConditionLock			*commandLock;
	msg_header_t			commandMessage;
	int						commandType;
	int						commandErrno;
	int						target;
	msg_header_t			timeoutMessage;
	int						ioctlCmd;
	caddr_t					ioctlData;
	struct uio				*uio;
	unsigned int			readFinished;

	/*
	 * IEEE-488 multiline messages
	 */
	unsigned char			ieee488Commands[40];
	int						ieee488CommandCount;
	int						ieee488CommandIndex;
	int						lastTalker;
	int						lastListener;
	
	/*
	 * I/O request state
	 */
	enum {
			IOS_IDLE,
			IOS_COMMAND,
			IOS_DATA,
			IOS_SPOLL,
			IOS_PPOLL,
	}						ioState;
	int						timeLimit;

	/*
	 * Software version of ISR flags
	 */
	BOOL					softCOflag;
	BOOL					softSRQIflag;
}

@end
