/*
 * NEC uPD7210 GPIB interface chip definitions
 * Copyright (C) 1994  W. Eric Norum
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * W. Eric Norum
 * Saskatchewan Accelerator Laboratory
 * University of Saskatchewan
 * 107 North Road
 * Saskatoon, Saskatchewan, CANADA
 * S7N 5C6
 * 
 * eric@skatter.usask.ca
 */

/*
 * Input registers
 */
#define UPDIN_DI			0
#define UPDIN_ISR1			1
#define UPDIN_ISR2			2
#define UPDIN_SPS			3
#define UPDIN_ADS			4
#define UPDIN_CPT			5
#define UPDIN_ADR0			6
#define UPDIN_ADR1			7

/*
 * Output registers
 */
#define UPDOUT_BO			0
#define UPDOUT_IMR1			1
#define UPDOUT_IMR2			2
#define UPDOUT_SPM			3
#define UPDOUT_AMR			4
#define UPDOUT_AUX			5
#define UPDOUT_ADR01		6
#define UPDOUT_EOS			7

/*
 * Interrupt Status Registers
 */
#define UPD_ISR1_CPT		0x80	/* Command Pass Through */
#define UPD_ISR1_APT		0x40	/* Address Pass Through */
#define UPD_ISR1_DET		0x20	/* Device Trigger */
#define UPD_ISR1_END		0x10	/* Data Block End (EOI or EOS) */
#define UPD_ISR1_DEC		0x08	/* Device Clear */
#define UPD_ISR1_ERR		0x04	/* Error (Byte Out data lost) */
#define UPD_ISR1_DO			0x02	/* Byte Out Register Ready */
#define UPD_ISR1_DI			0x01	/* Data In Register Ready */

#define UPD_ISR2_INT		0x80	/* Interrupt Pin */
#define UPD_ISR2_SRQI		0x40	/* Service Request */
#define UPD_ISR2_LOK		0x20	/* Lockout state (non interrupt) */
#define UPD_ISR2_REM		0x10	/* Remote state (non interrupt) */
#define UPD_ISR2_CO			0x08	/* Command Output ready */
#define UPD_ISR2_LOKC		0x04	/* Lockout state change */
#define UPD_ISR2_REMC		0x02	/* Remote state change */
#define UPD_ISR2_ADSC		0x01	/* Address status change

/*
 * Interrupt Mask Registers
 */
#define UPD_IMR1_CPT		0x80	/* Command Pass Through */
#define UPD_IMR1_APT		0x40	/* Address Pass Through */
#define UPD_IMR1_DET		0x20	/* Device Trigger */
#define UPD_IMR1_END		0x10	/* Data Block End (EOI or EOS) */
#define UPD_IMR1_DEC		0x08	/* Device Clear */
#define UPD_IMR1_DO			0x02	/* Byte Out Register Ready */
#define UPD_IMR1_DI			0x01	/* Data In Register Ready */

#define UPD_IMR2_SRQI		0x40	/* Service Request */
#define UPD_IMR2_DMAO		0x20	/* Enable DMA output requests */
#define UPD_IMR2_DMAI		0x10	/* Enable DMA input requests */
#define UPD_IMR2_CO			0x08	/* Command Output ready */
#define UPD_IMR2_LOKC		0x04	/* Lockout state change */
#define UPD_IMR2_REMC		0x02	/* Remote state change */
#define UPD_IMR2_ADSC		0x01	/* Address status change

/*
 * Address Status Register
 */
#define UPD_ADS_CIC			0x80	/* Controller In Charge */
#define UPD_ADS_ATNBAR		0x40	/* Device in DATA mode (ATN not asserted) */
#define UPD_ADS_SPMS		0x20	/* Device in SPMS state */
#define UPD_ADS_LPAS		0x10	/* Device in LPAS state */
#define UPD_ADS_TPAS		0x08	/* Device in TPAS state */
#define UPD_ADS_LA			0x04	/* Device in LADS state */
#define UPD_ADS_TA			0x02	/* Device in TADS state */
#define UPD_ADS_MJMN		0x01	/* 1-minor address rcvd, 0-major */

/*
 * Address Mode Register
 */
#define UPD_AMR_TON			0x80	/* Talk-only mode */
#define UPD_AMR_LON			0x40	/* Listen-only mode */
#define UPD_AMR_TRM1		0x20	/* T/R2,3 pin function mode bit 1 */
#define UPD_AMR_TRM0		0x10	/* T/R2,3 pin function mode bit 0 */
#define UPD_AMR_ADM1		0x02	/* Address mode bit 1 */
#define UPD_AMR_ADM0		0x01	/* Address mode bit 0 */

/*
 * Address registers
 */
#define UPD_ADR0_DT0		0x40	/* Disable talker address */
#define UPD_ADR0_DL0		0x20	/* Disable listener address */
#define UPD_ADR0_AD5_0		0x10	/* Address register 0 bit 5 */
#define UPD_ADR0_AD4_0		0x08	/* Address register 0 bit 4 */
#define UPD_ADR0_AD3_0		0x04	/* Address register 0 bit 3 */
#define UPD_ADR0_AD2_0		0x02	/* Address register 0 bit 2 */
#define UPD_ADR0_AD1_0		0x01	/* Address register 0 bit 1 */

#define UPD_ADR1_EOI		0x80	/* State of EOI line */
#define UPD_ADR1_DT0		0x40	/* Disable talker address */
#define UPD_ADR1_DL0		0x20	/* Disable listener address */
#define UPD_ADR1_AD5_1		0x10	/* Address register 1 bit 5 */
#define UPD_ADR1_AD4_1		0x08	/* Address register 1 bit 4 */
#define UPD_ADR1_AD3_1		0x04	/* Address register 1 bit 3 */
#define UPD_ADR1_AD2_1		0x02	/* Address register 1 bit 2 */
#define UPD_ADR1_AD1_1		0x01	/* Address register 1 bit 1 */

#define UPD_ADR01_ARS		0x80	/* Address Register Select */
#define UPD_ADR01_DT		0x40	/* Disable talker address */
#define UPD_ADR01_DL		0x20	/* Disable listener address */
#define UPD_ADR01_AD5		0x10	/* Address bit 5 */
#define UPD_ADR01_AD4		0x08	/* Address bit 4 */
#define UPD_ADR01_AD3		0x04	/* Address bit 3 */
#define UPD_ADR01_AD2		0x02	/* Address bit 2 */
#define UPD_ADR01_AD1		0x01	/* Address bit 1 */

/*
 * Auxiliary Mode Register Commands
 */
#define UPD_AUX_PON			0x00		/* Execute power-on */
#define UPD_AUX_CR			0x02		/* Chip reset */
#define UPD_AUX_FHS			0x03		/* Finish Handshake */
#define UPD_AUX_TRIG		0x04		/* Trigger */
#define UPD_AUX_RTL			0x05		/* Return To Local */
#define UPD_AUX_SEOI		0x06		/* Send END with next data */
#define UPD_AUX_NVID		0x07		/* Finish APT, Other Secondary Address */
#define UPD_AUX_VID			0x0F		/* Finish APT, My Secondary Address */
#define UPD_AUX_SPP			0x09		/* Set Parallel Poll Flag */
#define UPD_AUX_CPP			0x01		/* Clear Parallel Poll Flag */
#define UPD_AUX_GTS			0x10		/* Go To Standby */
#define UPD_AUX_TCA			0x11		/* Take Control Asynchronously */
#define UPD_AUX_TCS			0x12		/* Take Control Synchronously */
#define UPD_AUX_TCSE		0x1A		/* TCS at end of data transfer */
#define UPD_AUX_LTN			0x13		/* Listen */
#define UPD_AUX_LTNC		0x1B		/* Listen in continuous mode */
#define UPD_AUX_LUN			0x1C		/* Local Unlisten */
#define UPD_AUX_EPP			0x1D		/* Execute Parallel Poll */
#define UPD_AUX_SIFC		0x1E		/* Set IFC */
#define UPD_AUX_CIFC		0x16		/* Clear IFC */
#define UPD_AUX_SREN		0x1F		/* Set REN */
#define UPD_AUX_CREN		0x17		/* Clear REN */
#define UPD_AUX_DSC			0x14		/* Disable System Control */

/*
 * Auxiliary Mode Hidden Registers
 */
#define UPD_AUX_FREQ(x)		(0x20|(x))	/* Set reference frequency x */
#define UPD_AUX_A(x)		(0x80|(x))	/* Auxiliary register A */
#define UPD_AUX_B(x)		(0xA0|(x))	/* Auxiliary register B */
#define UPD_AUX_E(x)		(0xC0|(x))	/* Auxiliary register E */
