/*
 * Post-Load command for KPC488 driver.   Installs character special files.
 * Copyright (C) 1994  W. Eric Norum
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * W. Eric Norum
 * Saskatchewan Accelerator Laboratory
 * University of Saskatchewan
 * 107 North Road
 * Saskatoon, Saskatchewan, CANADA
 * S7N 5C6
 * 
 * eric@skatter.usask.ca
 */

#import <driverkit/IODeviceMaster.h>
#import <driverkit/IODevice.h>
#import <driverkit/IOConfigTable.h>
#import <errno.h>
#import <libc.h>
#import <stdio.h>

#define KPC488DeviceName				"KPC488"
#define IEEE488_DEVICE_DIRECTORY		"/dev/ieee488"
#define IEEE488_DEVICE_NAME_TEMPLATE	IEEE488_DEVICE_DIRECTORY "/%02d"

#define CONTROLLER_KEY	"Controller GPIB Address"

BOOL
makeDeviceEntry (int major, int minor, int controllerAddress)
{
	char cbuf[100];

	sprintf (cbuf, IEEE488_DEVICE_NAME_TEMPLATE, minor);

	/*
	 * Unlink the old character special file
	 */
	(void)unlink (cbuf);

	/*
	 * For non-controller addresses, create the new character special file
	 */
	if (minor != controllerAddress) {
		if (mknod (cbuf, S_IFCHR | 0666, makedev (major, minor)) != 0) {
			fprintf (stderr, "Can't create `%s': %s\n", cbuf, strerror (errno));
			return NO;
		}
	}
	return YES;
}

int
main (int argc, char **argv)
{	
	id deviceMaster = [IODeviceMaster new];
	IOObjectNumber objectNum;
	IOString kind;
	IOReturn rtn;
	int minor;
	unsigned int count;
	unsigned int major, controllerAddress;

	rtn = [deviceMaster lookUpByDeviceName:KPC488DeviceName
								objectNumber:&objectNum
								deviceKind:&kind];
	if (rtn != IO_R_SUCCESS) {
		fprintf (stderr, "Error connecting to driver: %s.\n", [IODevice stringFromReturn:rtn]);
		return 1;
	}

	count = 1;
	rtn = [deviceMaster getIntValues:&major
						forParameter:"IOMajorDevice"
						objectNumber:objectNum
						count:&count];
	if ((rtn != IO_R_SUCCESS) || (count != 1)) {
		fprintf (stderr, "Error getting major device number:%s.\n", [IODevice stringFromReturn : rtn]);
	}

	count = 1;
	rtn = [deviceMaster getIntValues:&controllerAddress
						forParameter:CONTROLLER_KEY
						objectNumber:objectNum
						count:&count];
	if ((rtn != IO_R_SUCCESS) || (count != 1)) {
		fprintf (stderr, "Error getting IEEE-488 controller address:%s.\n", [IODevice stringFromReturn : rtn]);
		return 1;
	}

	umask (0);
	(void)mkdir (IEEE488_DEVICE_DIRECTORY, 0755);
	for (minor = 0 ; minor <= 30 ; minor++) {
		if (makeDeviceEntry (major, minor, controllerAddress) == NO)
			return (1);
	}
	return 0;
}
