/*
 * Name: PostLoad.m
 * Description: Post-Load to install the /dev/pp?-Devices
 *	   Based on the code in the Example Parallel and Serial Port post
 *	   loaders supplied by NeXT.
 * Author: Christian Starkjohann <cs@ds1.kph.tuwien.ac.at>
 * Date: September 1 1994
 * Tabsize: 4
 *
 * Copyright (C) 1994  Christian Starkjohann
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <driverkit/IODeviceMaster.h>
#import <driverkit/IODevice.h>
#import <driverkit/IOConfigTable.h>
#import <errno.h>
#import <libc.h>
#import <stdio.h>
#ifdef Use_syslog
# import <syslog.h>
#else
# undef LOG_CRIT
# define LOG_CRIT	stderr
# undef LOG_WARNING
# define LOG_WARNING	stderr
# undef LOG_DEBUG
# define LOG_DEBUG	stderr
# undef syslog
# define syslog		fprintf
#endif

#import "../config.h"

		
int main(int argc, char **argv)
{
id 				deviceMaster = [IODeviceMaster new];
IOObjectNumber 	objectNum;
IOString 		kind;
IOReturn 		rtn;
unsigned 		array[5], count = 5, majorNum;
int				i, instances, dev;
char			nodename[512];


	rtn = [deviceMaster lookUpByDeviceName : DEVICENAME "0"
				  objectNumber : &objectNum
				    deviceKind : &kind];
	if (rtn) {
		syslog(LOG_CRIT,"Error contacting %s (%s)\n",
			DEVICENAME "0", [IODevice stringFromReturn : rtn]);
		exit(1);	
	}
	rtn = [deviceMaster getIntValues : array
			    forParameter : "GeneralInfo"
			    objectNumber : objectNum
			           count : &count];
	if(rtn || count < 3) {
		syslog(LOG_WARNING,"Error performing Parameter request (%s)\n",
			[IODevice stringFromReturn : rtn]);
		exit(1);	
	}
	majorNum = array[0];
	instances = array[2];
	for(i=0;i<instances;i++){
		sprintf(nodename, "/dev/pp%d", i);
		unlink(nodename);
		dev = makedev(majorNum, i);
		if(mknod(nodename, (0666 | S_IFCHR), dev))
			syslog(LOG_WARNING,"Can\'t create %s\r\n", nodename);
			/* Lets just rub it in ... */
		if(chmod(nodename,0666))
			syslog(LOG_WARNING,"Can\'t ensure %s is +rw\r\n", nodename);
	}
	return 0;
}
